function wp_gl.init()
	local grenade_launcher = load_particle("grenade.obj")
	local grenade = load_particle("kgrenade.obj")
	local mortar = load_particle("mortar.obj")
	local rifle = load_particle("rifle_bullet.obj")
	local gauss = load_particle("gauss_bullet_that_makes_hurt.obj")

	local n2fix = function( num, prec )
		return math.floor( num * (2^prec) )
	end

	local f2num = function( fix, prec )
		return fix / ( 2^prec )
	end

	local function putObject( object, objtype, x, y, xspd, yspd )
		local ix, iy = object:pos()
		local ixspd, iyspd = object:spd()
		
		object:set_pos( x, y )
		object:set_spd( xspd, yspd )
		
		object:shoot(objtype, 1, 0, 0, 1 )
		
		object:set_pos( ix, iy )
		object:set_spd( ixspd, iyspd )
	end

	local function shootWeapon( worm, x, y, xspd, yspd, type ) -- x and y are fixed point with 8 bit precision, xspd and yspd are fixed point with 16 bit precision!
		if type == "grenade_launcher" then
			putObject( worm, grenade_launcher, f2num(x, 8), f2num(y, 8), f2num(xspd,16), f2num(yspd, 16) )
		end
		if type == "grenade" then
			putObject( worm, grenade, f2num(x, 8), f2num(y, 8), f2num(xspd,16), f2num(yspd, 16) )
		end
		if type == "mortar" then
			putObject( worm, mortar, f2num(x, 8), f2num(y, 8), f2num(xspd,16), f2num(yspd, 16) )
		end
		if type == "rifle" then
			putObject( worm, rifle, f2num(x, 8), f2num(y, 8), f2num(xspd,16), f2num(yspd, 16) )
		end
		if type == "gauss" then
			putObject( worm, gauss, f2num(x, 8), f2num(y, 8), f2num(xspd,16), f2num(yspd, 16) )
		end
	end
	
	local glShootEvent = network_worm_event( "glshot", 
		function(event, worm, data)
			local x = data:get_int()
			local y = data:get_int()
			local vx = data:get_int()
			local vy = data:get_int()
			shootWeapon( worm, x, y, vx, vy, "grenade_launcher" ) 
		end
	)
	
	local grnShootEvent = network_worm_event( "glrshot", 
		function(event, worm, data)
			local x = data:get_int()
			local y = data:get_int()
			local vx = data:get_int()
			local vy = data:get_int()
			shootWeapon( worm, x, y, vx, vy, "grenade" ) 
		end
	)
	
	local mortarShootEvent = network_worm_event( "moshot", 
		function(event, worm, data)
			local x = data:get_int()
			local y = data:get_int()
			local vx = data:get_int()
			local vy = data:get_int()
			shootWeapon( worm, x, y, vx, vy, "mortar" ) 
		end
	)
	
	local rifleShootEvent = network_worm_event( "rishot", 
		function(event, worm, data)
			local x = data:get_int()
			local y = data:get_int()
			local vx = data:get_int()
			local vy = data:get_int()
			shootWeapon( worm, x, y, vx, vy, "rifle" ) 
		end
	)
	
	local gaussShootEvent = network_worm_event( "gashot", 
		function(event, worm, data)
			local x = data:get_int()
			local y = data:get_int()
			local vx = data:get_int()
			local vy = data:get_int()
			shootWeapon( worm, x, y, vx, vy, "gauss" ) 
		end
	)

	function wp_gl.shoot( worm )
		if AUTH then
			local wx, wy = worm:pos()
			local wvx, wvy = worm:spd()
			local ang = worm:angle()
			local vx,vy
			if worm:current_weapon():type():name() == "Grenade Launcher" then
				vx, vy = angle_vector( ang, 1.85 )
			end
			if worm:current_weapon():type():name() == "Grenade" then
				vx, vy = angle_vector( ang, 1.65 )
			end
			if worm:current_weapon():type():name() == "Mortar" then
				vx, vy = angle_vector( ang, 2.52 )
			end
			if worm:current_weapon():type():name() == "Rifle" then
				vx, vy = angle_vector( ang, 1 )
			end
			if worm:current_weapon():type():name() == "Gauss Gun" or worm:current_weapon():type():name() == "Gauss Gun " then
				vx, vy = angle_vector( ang, 2 )
			end
			
			
			-- Motion inheritance:
			if worm:current_weapon():type():name() == "Grenade" or worm:current_weapon():type():name() == "Grenade Launcher" then
				MotionInheritance = 0.5882
				vx = vx + MotionInheritance * wvx
				vy = vy + MotionInheritance * wvy
			end
				
				-- Get fixed point values ( necesary since there is no add_float binding :P )
				local fx, fy, fvx, fvy = n2fix( wx, 8 ), n2fix(wy, 8), n2fix(vx, 16), n2fix(vy, 16)
				
			--Send the message
			local b = new_bitstream()
			b:add_int(fx)
			b:add_int(fy)
			b:add_int(fvx)
			b:add_int(fvy)
				
				--Shoot the weapon
			if worm:current_weapon():type():name() == "Grenade Launcher" then
				shootWeapon( worm, fx, fy, fvx, fvy, "grenade_launcher" )
				glShootEvent:send( worm, b, 0, SendMode.ReliableUnordered, RepRule.Auth2All )
			end
			if worm:current_weapon():type():name() == "Grenade" then
				shootWeapon( worm, fx, fy, fvx, fvy, "grenade" )
				grnShootEvent:send( worm, b, 0, SendMode.ReliableUnordered, RepRule.Auth2All )
			end
			if worm:current_weapon():type():name() == "Mortar" then
				shootWeapon( worm, fx, fy, fvx, fvy, "mortar" )
				mortarShootEvent:send( worm, b, 0, SendMode.ReliableUnordered, RepRule.Auth2All )
			end
			if worm:current_weapon():type():name() == "Rifle" then
				shootWeapon( worm, fx, fy, fvx, fvy, "rifle" )
				rifleShootEvent:send( worm, b, 0, SendMode.ReliableUnordered, RepRule.Auth2All )
			end
			if worm:current_weapon():type():name() == "Gauss Gun" or worm:current_weapon():type():name() == "Gauss Gun " then
				shootWeapon( worm, fx, fy, fvx, fvy, "gauss" )
				gaussShootEvent:send( worm, b, 0, SendMode.ReliableUnordered, RepRule.Auth2All )
			end
		end
	end
	
end -- wp_gl.init()


	--[[ ]]
